@@ -7,7 +7,10 @@ from string import Template
 
 import click
 
+from uipath._cli._utils._constants import UIPATH_PROJECT_ID
+
 from ..telemetry import track
+from ..telemetry._constants import _PROJECT_KEY, _TELEMETRY_CONFIG_FILE
 from ._utils._console import ConsoleLogger
 from ._utils._project_files import (
     ensure_config_file,
@@ -23,6 +26,31 @@ console = ConsoleLogger()
 schema = "https://cloud.uipath.com/draft/2024-12/entry-point"
 
 
+def get_project_id() -> str:
+    """Get project ID from telemetry file if it exists, otherwise generate a new one.
+
+    Returns:
+        Project ID string (either from telemetry file or newly generated).
+    """
+    # first check if this is a studio project
+    if os.getenv(UIPATH_PROJECT_ID, None):
+        return os.getenv(UIPATH_PROJECT_ID)
+
+    telemetry_file = os.path.join(".uipath", _TELEMETRY_CONFIG_FILE)
+
+    if os.path.exists(telemetry_file):
+        try:
+            with open(telemetry_file, "r") as f:
+                telemetry_data = json.load(f)
+                project_id = telemetry_data.get(_PROJECT_KEY)
+                if project_id:
+                    return project_id
+        except (json.JSONDecodeError, IOError):
+            pass
+
+    return str(uuid.uuid4())
+
+
 def get_project_version(directory):
     toml_path = os.path.join(directory, "pyproject.toml")
     if not os.path.exists(toml_path):
@@ -40,7 +68,7 @@ def validate_config_structure(config_data):
 
 
 def generate_operate_file(entryPoints, dependencies=None):
-    project_id = str(uuid.uuid4())
+    project_id = get_project_id()
 
     first_entry = entryPoints[0]
     file_path = first_entry["filePath"]
@@ -225,7 +253,7 @@ def pack_fn(
         z.writestr(f"{projectName}.nuspec", nuspec_content)
         z.writestr("_rels/.rels", rels_content)
 
-        files = files_to_include(config_data, directory)
+        files = files_to_include(config_data, directory, include_uv_lock)
 
         for file in files:
             if file.is_binary:
@@ -247,17 +275,6 @@ def pack_fn(
                         with open(file.file_path, "r", encoding="latin-1") as f:
                             z.writestr(f"content/{file.relative_path}", f.read())
 
-        # Handle optional files, conditionally including uv.lock
-        if include_uv_lock:
-            file_path = os.path.join(directory, "uv.lock")
-            if os.path.exists(file_path):
-                try:
-                    with open(file_path, "r", encoding="utf-8") as f:
-                        z.writestr(f"content/{file}", f.read())
-                except UnicodeDecodeError:
-                    with open(file_path, "r", encoding="latin-1") as f:
-                        z.writestr(f"content/{file}", f.read())
-
 
 def display_project_info(config):
     max_label_length = max(
