@@ -1,4 +1,5 @@
 # Environment variables
+DOTENV_FILE = ".env"
 ENV_BASE_URL = "UIPATH_URL"
 ENV_UNATTENDED_USER_ACCESS_TOKEN = "UNATTENDED_USER_ACCESS_TOKEN"
 ENV_UIPATH_ACCESS_TOKEN = "UIPATH_ACCESS_TOKEN"
@@ -18,6 +19,7 @@ HEADER_USER_AGENT = "x-uipath-user-agent"
 HEADER_TENANT_ID = "x-uipath-tenantid"
 HEADER_INTERNAL_TENANT_ID = "x-uipath-internal-tenantid"
 HEADER_JOB_KEY = "x-uipath-jobkey"
+HEADER_SW_LOCK_KEY = "x-uipath-sw-lockkey"
 
 # Data sources
 ORCHESTRATOR_STORAGE_BUCKET_DATA_SOURCE = (
@@ -29,3 +31,6 @@ TEMP_ATTACHMENTS_FOLDER = "uipath_attachments"
 
 # LLM models
 COMMUNITY_agents_SUFFIX = "-community-agents"
+
+# File names
+UIPATH_CONFIG_FILE = "uipath.json"
